# Generated by Django 2.2.28 on 2023-12-05 14:04
# flake8: noqa
from django.db import (
    migrations,
)


class Migration(migrations.Migration):

    dependencies = [
        ('edu_rdm_integration', '0006_request_status_data'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='uploadstatusrequestlog',
            name='status',
        ),
        migrations.DeleteModel(
            name='UploadStatus',
        ),
    ]
